# Remote font importing basically works Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output NULL --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output NULL --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" # font_google(local = TRUE) basically works Code tagz$html Output
# Special named args in font_collection Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] "" --- Code tagz$html Output
--- Code tagz$dependencies[[1]]$head Output [1] ""